import { nameof } from 'ts-simple-nameof';
import { TypedEvent } from './TypedEvent';
import { LoggerService } from './Services/Logging/LoggerService';
var DependencyContainer = (function () {
    function DependencyContainer() {
        this._services = {};
        this._serviceRegistrationEvent = new TypedEvent();
    }
    Object.defineProperty(DependencyContainer, "Instance", {
        get: function () {
            if (DependencyContainer._instance != null)
                return DependencyContainer._instance;
            DependencyContainer._instance = new DependencyContainer();
            this.Log = new LoggerService();
            this.Log.Warning("Initializing DependencyContainer");
            DependencyContainer._instance.RegisterService(this.Log);
            this.IsInitialized = true;
            return DependencyContainer._instance;
        },
        enumerable: false,
        configurable: true
    });
    DependencyContainer.prototype.RegisterService = function (service) {
        var _a;
        (_a = DependencyContainer.Log) === null || _a === void 0 ? void 0 : _a.Debug(DependencyContainer, "Registering new service: ".concat(service.GetServiceName()));
        this._services[service.GetServiceName()] = service;
    };
    DependencyContainer.prototype.GetService = function (classType) {
        var _a;
        var name = nameof(classType);
        (_a = DependencyContainer.Log) === null || _a === void 0 ? void 0 : _a.Debug(DependencyContainer, "Getting service [".concat(name, "] by GetService<T>()"));
        return this._services[name];
    };
    DependencyContainer.prototype.GetServiceAsync = function (classType, token) {
        var _this = this;
        var _a, _b;
        var name = nameof(classType);
        (_a = DependencyContainer.Log) === null || _a === void 0 ? void 0 : _a.Debug(DependencyContainer, "Waiting for service availability [".concat(name, "] by GetServiceAsync<T>()"));
        if (this._services.hasOwnProperty(name)) {
            (_b = DependencyContainer.Log) === null || _b === void 0 ? void 0 : _b.Debug(DependencyContainer, "Got service [".concat(name, "] by GetServiceAsync<T>()"));
            return Promise.resolve(this._services[name]);
        }
        return new Promise(function (resolve, reject) {
            var sub = function (service) {
                var _a;
                if ((service === null || service === void 0 ? void 0 : service.GetServiceName()) === name) {
                    (_a = DependencyContainer.Log) === null || _a === void 0 ? void 0 : _a.Debug(DependencyContainer, "Got service [".concat(name, "] by GetServiceAsync<T>()"));
                    _this._serviceRegistrationEvent.Unsubscribe(sub);
                    resolve(service);
                }
            };
            var onCancel = function () {
                var _a;
                _this._serviceRegistrationEvent.Unsubscribe(sub);
                token === null || token === void 0 ? void 0 : token.CancellationEvent.Unsubscribe(onCancel);
                (_a = DependencyContainer.Log) === null || _a === void 0 ? void 0 : _a.Debug(DependencyContainer, "Cancelled service [".concat(name, "] retrieval by GetServiceAsync<T>()"));
                reject(new Error("Operation cancelled!"));
            };
            token === null || token === void 0 ? void 0 : token.CancellationEvent.Subscribe(onCancel);
            _this._serviceRegistrationEvent.Subscribe(sub);
        });
    };
    DependencyContainer.prototype.GetServiceWithTimeoutAsync = function (classType, timeout) {
        var _this = this;
        var _a, _b;
        var name = nameof(classType);
        (_a = DependencyContainer.Log) === null || _a === void 0 ? void 0 : _a.Debug(DependencyContainer, "Waiting for service[".concat(name, "] availability by GetServiceWithTimeoutAsync<T>()"));
        if (this._services.hasOwnProperty(name)) {
            (_b = DependencyContainer.Log) === null || _b === void 0 ? void 0 : _b.Debug(DependencyContainer, "Got service [".concat(name, "] by GetServiceWithTimeoutAsync<T>()"));
            return Promise.resolve(this._services[name]);
        }
        return new Promise(function (resolve, reject) {
            var timeoutId = 0;
            var sub = function (service) {
                var _a;
                if ((service === null || service === void 0 ? void 0 : service.GetClassName()) === name) {
                    (_a = DependencyContainer.Log) === null || _a === void 0 ? void 0 : _a.Debug(DependencyContainer, "Got service [".concat(name, "] by GetServiceWithTimeoutAsync<T>()"));
                    window.clearTimeout(timeoutId);
                    _this._serviceRegistrationEvent.Unsubscribe(sub);
                    resolve(service);
                }
            };
            var onTimeout = function () {
                var _a;
                window.clearTimeout(timeoutId);
                _this._serviceRegistrationEvent.Unsubscribe(sub);
                (_a = DependencyContainer.Log) === null || _a === void 0 ? void 0 : _a.Error(DependencyContainer, "Failed to get service [".concat(name, "] in time by GetServiceWithTimeoutAsync<T>()"));
                reject(new Error("Operation cancelled!"));
            };
            timeoutId = window.setTimeout(onTimeout, timeout);
            _this._serviceRegistrationEvent.Subscribe(sub);
        });
    };
    DependencyContainer.IsInitialized = false;
    return DependencyContainer;
}());
export { DependencyContainer };
//# sourceMappingURL=DependencyContainer.js.map