import { IDeviceTask } from './IDeviceTask';
import { DeviceTaskExecutionResult } from './TaskExecutionResult';
import { IDeviceChannel } from './IDeviceChannel';
import { FieldChangeResult } from '../FieldChangeResult';
import { ITypedEvent } from '../../TypedEvent';
import { IDeviceState } from './IDeviceState';
export interface IDevice {
    OnDeviceStateChangedEvent(): ITypedEvent<IDeviceState>;
    get Guid(): string;
    get Name(): string;
    get SerialNumber(): string;
    get ModelGuid(): string;
    get ManufacturerGuid(): string;
    get IsVirtual(): boolean;
    get IsEnabled(): boolean;
    get DeviceState(): DeviceState;
    get DeviceType(): DeviceType;
    get CommunicationWay(): CommunicationWay;
    get AvailableTasks(): string[];
    get AvailableTaskTypes(): DeviceTaskType[];
    get AvailableResponseTypes(): DeviceResponseType[];
    get ChannelsAmount(): number;
    get Channels(): IDeviceChannel[];
    ChangeDeviceNameAsync(name: string): Promise<FieldChangeResult>;
    ExecuteTaskAsync(task: IDeviceTask, channel: number): Promise<DeviceTaskExecutionResult>;
}
export declare enum DeviceTaskType {
    Unknown = ""
}
export declare enum DeviceResponseType {
    Unknown = ""
}
export declare enum DeviceState {
    NotResponding = 0,
    Working = 1,
    Broken = 2,
    FirmareUpgradeMode = 3
}
export declare enum CommunicationWay {
    TwoWay = 1,
    OneWay = 2,
    ConditionalTwoWay = 3
}
export declare enum DeviceType {
    Unknown = -1,
    Multipurpose = 0,
    Controller = 1,
    Driver = 2,
    Receiver = 3,
    Remote = 4,
    Sensor = 5,
    Scene = 6
}
